@@ -15,9 +15,13 @@
 #' or fixed, \code{"max"}.
 #' @param validation type of validation for \code{plsr}. The default is "CV". If more
 #' than one set of CV segments is wanted, use a vector of lenth two, e.g. \code{c("CV",5)}.
+#' @param fixed vector of indeces for compulsory/fixed variables that should always be
+#' included in the modelling.
 #' @param newy validation response for RMSEP/error computations.
 #' @param newX validation predictors for RMSEP/error computations.
 #' @param segments see \code{mvr} for documentation of segment choices.
+#' @param plsType Type of PLS model, "plsr" or "cppls".
+#' @param Y.add Additional response for CPPLS, see \code{plsType}.
 #' @param ... additional arguments for \code{plsr} or \code{cvsegments}.
 #' @param x object of class \code{shaved} for plotting or printing.
 #' @param what plot type. Default = "error". Alternative = "spectra".
@@ -54,9 +58,9 @@
 #'
 #' @importFrom progress progress_bar
 #' @export
-shaving <- function(y, X, ncomp = 10, method = c("SR", "VIP", "sMC", "LW", "RC"), prop = 0.2, min.left = 1,
-                    comp.type = c("CV", "max"), validation = c("CV", 1), newy = NULL, newX = NULL,
-                    segments = 10, ...){
+shaving <- function(y, X, ncomp = 10, method = c("SR", "VIP", "sMC", "LW", "RC"), prop = 0.2, min.left = 2,
+                    comp.type = c("CV", "max"), validation = c("CV", 1), fixed = integer(0), newy = NULL, newX = NULL,
+                    segments = 10, plsType = "plsr", Y.add = NULL, ...){
   
   modeltype <- "prediction"
   if (is.factor(y)) {
@@ -70,7 +74,7 @@ shaving <- function(y, X, ncomp = 10, method = c("SR", "VIP", "sMC", "LW", "RC")
   # Initialization
   n <- nrow(X)
   p <- ncol(X)
-  if(min.left < 0){ # Get minimum size
+  if(min.left < 1){ # Get minimum size
     min.left <- max(1, round(min.left * p))
   }
   comp.type <- comp.type[1]
@@ -79,12 +83,22 @@ shaving <- function(y, X, ncomp = 10, method = c("SR", "VIP", "sMC", "LW", "RC")
   }
   
   # Find the number of shaves
-  nrep <- 0
-  left <- p
+  nrep <- 1
+  nfix <- length(fixed)
+  if(nfix >= p)
+    stop('Too many variables are fixed.')
+  left <- p-nfix
   while(left > min.left){
     nrep <- nrep + 1
     left <- left - max(1, round(prop * left))
   }
+  rems <- integer(nrep)
+  left <- p-nfix
+  for(i in 1:nrep){
+    rems[i] <- left
+    left <- left - max(1, round(prop * left))
+  }
+  rems <- -diff(rems)
   left.vec <- 1:p # Vector of remaining variables
   
   # Prepare for expanded/repated segments
@@ -106,7 +120,7 @@ shaving <- function(y, X, ncomp = 10, method = c("SR", "VIP", "sMC", "LW", "RC")
   comps       <- integer(nrep)
   nvar        <- integer(nrep)
   variables   <- list()
-  left        <- p
+  # left        <- p
   
   # Shaving loop
   pb <- progress_bar$new(total = nrep, format = "  [:bar] :percent ")
@@ -114,21 +128,44 @@ shaving <- function(y, X, ncomp = 10, method = c("SR", "VIP", "sMC", "LW", "RC")
     nvar[i] <- length(left.vec)
     variables[[i]] <- left.vec
 
-    rem       <- max(1, round(prop * left))
-    left      <- left - rem
-    comp      <- min(n-1, left, ncomp)
-    data      <- data.frame(y = y, X = I(X[, left.vec, drop = FALSE]))
-    if(validation == "CV"){
-      pls       <- plsr(y ~ X, ncomp=comp, data=data, validation = validation, segments = segs, ...)
+    comp      <- min(n-1, nvar[i], ncomp)
+    if(is.null(Y.add)){
+      data      <- data.frame(y = y, X = I(X[, left.vec, drop = FALSE]))
     } else {
-      pls       <- plsr(y ~ X, ncomp=comp, data=data, validation = validation, ...)
+      data      <- data.frame(y = y, X = I(X[, left.vec, drop = FALSE]), Y.add = Y.add)
+    }
+    if(validation == "CV"){
+      if(plsType == "cppls"){
+        if(is.null(Y.add)){
+          pls       <- cppls(y ~ X, ncomp=comp, data=data, validation = validation, segments = segs, ...)
+          
+        } else {
+          pls       <- cppls(y ~ X, ncomp=comp, data=data, validation = validation, segments = segs, Y.add = Y.add, ...)
+        }
+      } else {
+        pls       <- plsr(y ~ X, ncomp=comp, data=data, validation = validation, segments = segs, ...)
+      }
+    } else {
+      if(plsType == "cppls"){
+        if(is.null(Y.add)){
+          pls       <- cppls(y ~ X, ncomp=comp, data=data, validation = validation, ...)
+        } else {
+          pls       <- cppls(y ~ X, ncomp=comp, data=data, validation = validation, Y.add = Y.add, ...)
+        }
+      } else {
+        pls       <- pls(y ~ X, ncomp=comp, data=data, validation = validation, ...)
+      }
     }
     # factor y
     if(comp.type == "CV"){
       if (modeltype == "prediction"){
         opt.comp <- which.min(pls$validation$PRESS[1,])
       } else if (modeltype == "classification"){
-        classes <- lda_from_pls_cv(pls, data$X, y.orig, comp)
+        if(is.null(Y.add)){
+          classes <- lda_from_pls_cv(pls, data$X, y.orig, comp)
+        } else {
+          classes <- lda_from_pls_cv(pls, data$X, y.orig, comp, Y.add)
+        }
         opt.comp <- which.max(colSums(classes==y.orig))
       }
     } else {
@@ -168,10 +205,21 @@ shaving <- function(y, X, ncomp = 10, method = c("SR", "VIP", "sMC", "LW", "RC")
     if(method == "RC"){
       weights <- RC(pls, opt.comp)
     }
-    left.vec <- left.vec[sort(order(weights)[-(1:rem)])]
+    if(i < nrep)
+      rem <- rems[i]
+    # rem       <- max(1, round(prop * (left-nfix)))
+    left      <- left - rem
+    if(nfix == 0){
+      left.vec <- left.vec[sort(order(weights)[-(1:rem)])]
+    } else {
+      left.vec1 <- left.vec[!(left.vec%in%fixed)]
+      weights1  <- weights[!(left.vec%in%fixed)]
+      left.vec1 <- left.vec1[sort(order(weights1)[-(1:rem)])]
+      left.vec  <- sort(c(left.vec1,fixed))
+    }
     pb$tick()
   }
-  obj <- list(min.error = min(error), min.red = which.min(error)-1, method = method, error = error, comps = comps, nvar = nvar, variables = variables, X = X)
+  obj <- list(min.error = min(error), min.red = index <- length(error)-which.min(rev(error)), method = method, error = error, comps = comps, nvar = nvar, variables = variables, X = X)
   class(obj) <- c("shaved", "list")
   obj
 }
@@ -189,7 +237,7 @@ plot.shaved <- function(x, y, what = c('error', 'spectra'), index = "min", log =
     with(x, {
       # Prepare
       if(is.character(index) && index == "min"){
-        index <- which.min(error)
+        index <- length(error)-which.min(rev(error))+1
       }
       X.NA <- X
       X.NA[, -variables[[index]]] <- NA
@@ -222,6 +270,6 @@ print.shaved <- function(x, ...){
   cat('Shaving using ', x$method, ':\n', sep = "")
   cat('Minimum error = ', (x$min.err), ', achieved after ', x$min.red,
       ' out of ', length(x$error)-1, ' reductions using ',
-      x$nvar[which.min(x$error)], ' variables.', sep = "")
+      x$nvar[length(x$error)-which.min(rev(x$error))+1], ' variables.', sep = "")
 }
 
